#include <stdio.h>
#include <stdlib.h>

#define N_BITS sizeof(int) * 8

// The left shift and right shift operators should not be used for negative numbers.
// The result is undefined behaviour if any of the operands is a negative number.
// For example results of both -1 << 1 and 1 << -1 is undefined.

// If the number is shifted more than its size, the behaviour is undefined.
// For example, 1 << 33 is undefined if integers are stored using 32 bits.

void print(const char*, unsigned int);

int main (void) {

	unsigned int op1 = 0xf0f0f0f0;
	unsigned int op2 = 0xff00ff00;

	print("arg1", op1);
	print("arg2", op2);
	print("~arg1", ~op1);
	print("a1 & a2", op1 & op2);
	print("a1 | a2", op1 | op2);
	print("a1 ^ a2", op1 ^ op2);
	print("a1 << 4", op1 << 4);
	print("a1 >> 4", op1 >> 4);
	printf("\n");

	print("1 & 2", 1 & 2);
	print("1 && 2", 1 && 2);

	return EXIT_SUCCESS;
}

void print(const char *s, unsigned int v) {

	unsigned char buf[N_BITS + 1];
	for (int position = N_BITS - 1; position >= 0; position--) {
		buf[position] = (v & 1) + '0';
		v >>= 1;
	}
	buf[N_BITS] = '\0';
	printf("%8s: %s\n", s, buf);
}

